function [ruta]=calcula_ruta(origen,destino,ficheroTIF,modo)

%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% 
% Funcin para calcular la ruta ms corta entre dos puntos de un mapa,
% empleando un algoritmo de bsqueda A*, y representar la ruta calculada en
% un mapa
%--------------------------------------------------------------------------
%
% SINTAXIS: [ruta]=calcula_ruta(origen,destino,ficheroTIF,modo)
%--------------------------------------------------------------------------
%
% INPUT: origen: Ciudad de la que parte la ruta
%        destino: Ciudad a la que llega la ruta
%        ficheroTIF: Fichero imagen que contiene el mapa sobre el que se 
%                    calcular la ruta
%        modo: Criterio de optimalidad de la ruta
%--------------------------------------------------------------------------
%
% OUTPUT: ruta: Sucesin de puntos [fila,columna] ptima entre dos puntos a
%               travs de un mapa
%--------------------------------------------------------------------------
%
%                                                   Trabajo de Fin de Grado
%               "Sistema de seguimiento y gestin de mercancas peligrosas"
%                                       Diego Cuervo Fernndez (08/04/2014)
%                                                             Matlab R2013a
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

%CONSTANTE FACTOR DE ESCALA CARACTERSTICO DEL MAPA
factor_escala=81.709626;

%CONSTANTES CON LAS VELOCIDADES DE CADA TIPO DE CARRETERA (en m/s) Y FICHERO CARR_TIPO
v_autovia=80/3.6;
v_nacional=60/3.6;
v_comarcal=50/3.6;
v_local=40/3.6;
carr_tipo='Carreteras_tipo.tif';

%Obtencion de coordenadas origen y destino
[fil_ini,col_ini]=lee_coordenadas(origen);
if(fil_ini == -1 || col_ini == -1)
    return
end

[fil_obj,col_obj]=lee_coordenadas(destino);
if(fil_obj == -1 || col_obj == -1)
    return
end

%Obtencion del parametro a optimizar
if(strcmp(modo,'distancia_minima')==1)
    param=1;
else 
    if(strcmp(modo,'tiempo_minimo')==1)
        param=2;
    else
        disp(sprintf('Error en el cuarto argumento de la funcion.\n Debe ser "tiempo_minimo" o "distancia_minima"'));
        return
    end
end

%Se carga el mapa de la imagen .TIF y se transforma en una imagen binaria
mapaRGB=imread(ficheroTIF);
mapa=binariza(mapaRGB);

%Se corrige el desfase por el uso del '0' como ndice vlido de la matriz
mapa(1:4981,1:6324)=mapa(2:4982,2:6325);

%Se carga el mapa con los tipos de carreteras
mapa_tipo=imread(carr_tipo);

%Se corrige el desfase por el uso del '0' como ndice vlido de la matriz
mapa_tipo(1:4981,1:6324)=mapa_tipo(2:4982,2:6325);

%Almacenamiento del tamao del mapa
[fila_max,columna_max]=size(mapa);

%Creacin de la estructura de datos de los nodos para el algoritmo
nodo(1).profundidad=0;
nodo(1).fila=fil_ini; 
nodo(1).columna=col_ini;
nodo(1).padre=0;  %ndice del padre del nodo
nodo(1).dist_recorrida=0; %Distancia recorrida en el nodo
nodo(1).tiempo_empleado=0;
nodo(1).siguiente=0;

nodos_expandidos=0;

if(param==1)
    %El coste de un nodo es la suma de la distancia recorrida ms la distancia
    %hasta la posicin objetivo
    nodo(1).coste=0+distancia(nodo(1).fila,nodo(1).columna,fil_obj, col_obj, factor_escala);
else
    %El coste del nodo ser ahora el tiempo empleado ms el menor tiempo
    %posible hasta posicin objetivo (en el mejor de los casos)
    nodo(1).coste=0+(distancia(nodo(1).fila,nodo(1).columna,fil_obj, col_obj, factor_escala))/v_autovia;
end

%Puntero al inicio de la lista
inicio=1;

%Indice para recorrer la lista de nodos
indice=1;

%Numero de nodos en la lista
num_nodos=1;

%Indicador de haber alcanzado el objetivo
exito=0;

tic
%Bucle principal del programa, que se acaba al alcanzar el xito o al quedarse sin nodos en la lista
while(exito==0 && indice ~= 0)
    
    %Se comprueba si el nodo actual es el objetivo
    exito=prueba_meta_map(nodo(indice).fila,nodo(indice).columna,fil_obj,col_obj);
    
    %Si no es el objetivo
    if(exito==0)
    
        %Llamada a la funcin para expandir el nodo para obtener sus hijos
        [hijo,num_hijos]=expandir_nodo(nodo(indice),indice,mapa,fila_max,columna_max,factor_escala,mapa_tipo,v_autovia,v_nacional,v_comarcal,v_local);
        nodos_expandidos=nodos_expandidos+1;
        
        %Bucle para recorrer el vector de hijos
        for j=1:num_hijos        
            
            if(param==1)
                %Clculo del coste para cada uno de los hijos (criterio de distancia)
                nuevo_coste= hijo(j).dist_recorrida+distancia(hijo(j).fila,hijo(j).columna,fil_obj, col_obj, factor_escala);
            else
                %Clculo del coste para cada uno de los hijos (critero de tiempo)
                nuevo_coste= hijo(j).tiempo_empleado+distancia(hijo(j).fila,hijo(j).columna,fil_obj, col_obj, factor_escala)/v_autovia;
            end
            
            %Cada uno de los hijos se asigna a un nuevo elemento de la lista de nodos
            nodo(num_nodos+1)=hijo(j);
                        
            %Llamada a la funcin para insertar en lista el nuevo nodo por orden de coste                                                
            [nodo,inicio,num_nodos]=insertar_en_lista(nuevo_coste,nodo,inicio,num_nodos);
           
            %Se marca con un '0' en el mapa el hijo para no volver a generarlo
            mapa(hijo(j).fila,hijo(j).columna)=0;
        end
        
        %Variacin del indice y del puntero al inicio para la siguiente iteracion
        indice=nodo(inicio).siguiente;
        inicio=indice;        
    end
end
t=toc;

%Una vez acabado el bucle, si se ha alcanzado la posicin objetivo
if(exito==1)
    
    %Clculo de la distancia total en km
    distancia_total=nodo(indice).dist_recorrida;
    horas_totales=floor(nodo(indice).tiempo_empleado/3600);
    minutos_totales=floor((nodo(indice).tiempo_empleado/3600-horas_totales)*60);
    segundos_totales=round(((nodo(indice).tiempo_empleado/3600-horas_totales)*60-minutos_totales)*60);
    
    %Se muestran los reultados obtenidos por pantalla
    disp(sprintf('xito alcanzado'));
    disp(sprintf('Distancia recorrida: %.2f kilometros',distancia_total/1000));
    disp(sprintf('Tiempo empleado: %d horas %d minutos %.d segundos',horas_totales,minutos_totales,segundos_totales));
    disp(sprintf('Profundidad de la solucion: %d', nodo(indice).profundidad));
    disp(sprintf('Nodos expandidos: %d', nodos_expandidos));
    disp(sprintf('Tiempo de ejecucin del algoritmo: %.2f segundos', t));

    %Se inicializa el puntero i para almacenar el vector ruta
    i=nodo(indice).profundidad+1;
    tamano_ruta=i;
    
    %Bucle para almacenar la ruta
    while(i>0)
       
        %Se almacena la ruta optima
        ruta(i).fila=nodo(indice).fila;
        ruta(i).columna=nodo(indice).columna;
        
        %Variacion del indice para la siguiente iteracion
        indice=nodo(indice).padre;
        
        %Actualizacin del puntero
        i=i-1;
    end        
    
    
    %Llamada a la funcin para representar la ruta calculada sobre el mapa
    muestra_ruta(ficheroTIF,ruta,tamano_ruta);
else
    
    %Si no se alcanza el objetivo, se advierte al usuario por pantalla 
    disp(sprintf('Exito no alcanzado :('));
end
