function [indice,recorrido] = calcula_siguiente_posicion_estricto(velocidad_kph,tiempo,ruta,indice,recorrido,profundidad)

%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% 
% Funcin para calcular la siguiente posicin de un camin dado una
% velocidad, un tiempo y una posicin dentro de una ruta
%--------------------------------------------------------------------------
%
% SINTAXIS: [indice,recorrido] = calcula_siguiente_posicion_estricto...
%           ...(velocidad_kph,tiempo,ruta,indice,recorrido,profundidad)
%--------------------------------------------------------------------------
%
% INPUT: velocidad_kph: Velocidad del camin en kilmetros por hora
%        tiempo: Tiempo para el cual se calcula la siguiente posicin
%        ruta: Ruta a seguir por el camin
%        indice: Posicin a cada momento del camin dentro de su ruta
%        recorrido: Distancia que el camin lleva recorrida dentro del
%                   pxel actual
%        profundidad: Nmero de puntos en la ruta del camin
%--------------------------------------------------------------------------
%
% OUTPUT: indice: Posicin calculada del camin dentro de su ruta
%         recorrido: Distancia que el camin lleva recorrida dentro del
%                    pxel actual
%--------------------------------------------------------------------------
%
%                                                   Trabajo de Fin de Grado
%               "Sistema de seguimiento y gestin de mercancas peligrosas"
%                                       Diego Cuervo Fernndez (19/04/2014)
%                                                             Matlab R2013a
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------


%Se convierte la velocidad a metros por segundo
velocidad_mps=velocidad_kph/3.6;

%Escala del mapa (81.709626 metros por fila o columna)
factor_escala=81.709626;

%Se comprueba si el camin est avanzando en diagonal o en vertical/horizontal
if(ruta(indice).fila==ruta(indice+1).fila || ruta(indice).columna==ruta(indice+1).columna)
    
    %Si se avanza en horizontal o vertical, la distancia restante hasta el prximo pxel
    %ser el factor de escala menos lo que ya llevamos recorrido
    restante=1*factor_escala-recorrido;
else
    
    %Si se avanza en diagonal, la distancia restante hasta el prximo pxel ser el 
    %factor de escala multiplicado por raz de 2 menos lo que ya llevamos recorrido
    restante=sqrt(2)*factor_escala-recorrido;
end

%Tiempo consumido en avanzar hasta el siguiente pxel
consumido=restante/velocidad_mps;

%Mientras el tiempo consumido no supere el tiempo total y no se llegue al final de la ruta
while(consumido<=tiempo && indice<profundidad-1);
    
    %Se avanza al pxel siguiente dentro de la ruta
    indice=indice+1;
    
    %Se calcula la distancia restante del pixel
    if(ruta(indice).fila==ruta(indice+1).fila || ruta(indice).columna==ruta(indice+1).columna)
        restante=1*factor_escala;
    else
        restante=sqrt(2)*factor_escala;
    end
    
    %Se actualiza el tiempo consumido
    consumido=consumido+restante/velocidad_mps;
end

%Si se llega al final de la ruta, se devuelve un -1 para indicarlo
if(indice==profundidad-1)
    recorrido=-1;
    
%Si se acaba el tiempo, se calcula el recorrido restante del pxel actual
else
    recorrido=restante-(consumido-tiempo)*velocidad_mps;
end