function [ruta]=genera_ruta_origenes_destinos(fil_ini,col_ini,fil_obj, col_obj, mapa,mapa_tipo,nfichero,param)

%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% 
% Funcin para calcular la ruta ms corta entre dos puntos de un mapa,
% empleando un algoritmo de bsqueda A*, y guardar la ruta calculada en un
% fichero de datos
%--------------------------------------------------------------------------
%
% SINTAXIS: [ruta]=genera_ruta_origenes_destinos(fil_ini,col_ini,...
%           ...fil_obj,col_obj,mapa,mapa_tipo,nfichero,param)
%--------------------------------------------------------------------------
%
% INPUT: fil_ini: Fila del mapa correspondiente a la posicin inicial
%        col_ini: Columna del mapa correspondiente a la posicin inicial
%        fil_obj: Fila del mapa correspondiente a la posicin objetivo
%        col_obj: Columna del mapa correspondiente a la posicin objetivo
%        mapa: Matriz binaria con los datos de las carreteras
%        mapa_tipo: Matriz con los tipos de carreteras
%        nfichero: Nombre del fichero en el que se almacenar la ruta
%        param: Variable de seleccin del parmetro a optimizar (si vale 1
%               optimiza distancia, si vale 2 optimiza tiempo)
%--------------------------------------------------------------------------
%
% OUTPUT: ruta: Sucesin de puntos [fila,columna] ptima entre dos puntos a
%               travs de un mapa
%--------------------------------------------------------------------------
%
%                                                   Trabajo de Fin de Grado
%               "Sistema de seguimiento y gestin de mercancas peligrosas"
%                                       Diego Cuervo Fernndez (16/04/2014)
%                                                             Matlab R2013a
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

%CONSTANTE FACTOR DE ESCALA CARACTERSTICO DEL MAPA
factor_escala=81.709626;

%CONSTANTES CON LAS VELOCIDADES DE CADA TIPO DE CARRETERA (en m/s) Y FICHERO CARR_TIPO
v_autovia=80/3.6;
v_nacional=60/3.6;
v_comarcal=50/3.6;
v_local=40/3.6;

%Almacenamiento del tamao del mapa
[fila_max,columna_max]=size(mapa);

%Creacin de la estructura de datos de los nodos para el algoritmo
nodo(1).profundidad=0;
nodo(1).fila=fil_ini;
nodo(1).columna=col_ini;
nodo(1).padre=0;  %ndice del padre del nodo
nodo(1).dist_recorrida=0; %Distancia recorrida en el nodo
nodo(1).tiempo_empleado=0;
nodo(1).siguiente=0;

if(param==1)
    %El coste de un nodo es la suma de la distancia recorrida ms la distancia
    %hasta la posicin objetivo
    nodo(1).coste=0+distancia(nodo(1).fila,nodo(1).columna,fil_obj, col_obj, factor_escala);
else
    %El coste del nodo ser ahora el tiempo empleado ms el menor tiempo
    %posible hasta posicin objetivo (en el mejor de los casos)
    nodo(1).coste=0+(distancia(nodo(1).fila,nodo(1).columna,fil_obj, col_obj, factor_escala))/v_autovia;
end

%Puntero al inicio de la lista
inicio=1;

%Indice para recorrer la lista de nodos
indice=1;

%Numero de nodos en la lista
num_nodos=1;

%Indicador de haber alcanzado el objetivo
exito=0;

%Bucle principal del programa, que se acaba al alcanzar el xito o al quedarse sin nodos en la lista
while(exito==0 && indice ~= 0)
    %Se comprueba si el nodo actual es el objetivo
    exito=prueba_meta_map(nodo(indice).fila,nodo(indice).columna,fil_obj,col_obj);
    
    %Si no es el objetivo
    if(exito==0)
        
        %Llamada a la funcin para expandir el nodo para obtener sus hijos
        [hijo,num_hijos]=expandir_nodo(nodo(indice),indice,mapa,fila_max,columna_max,factor_escala,mapa_tipo,v_autovia,v_nacional,v_comarcal,v_local);

        %Bucle para recorrer el vector de hijos
        for j=1:num_hijos        
            
           if(param==1)
                %Clculo del coste para cada uno de los hijos (criterio de distancia)
                nuevo_coste= hijo(j).dist_recorrida+distancia(hijo(j).fila,hijo(j).columna,fil_obj, col_obj, factor_escala);
            else
                %Clculo del coste para cada uno de los hijos (critero de tiempo)
                nuevo_coste= hijo(j).tiempo_empleado+distancia(hijo(j).fila,hijo(j).columna,fil_obj, col_obj, factor_escala)/v_autovia;
           end
            
            %Cada uno de los hijos se asigna a un nuevo elemento de la lista de nodos
            nodo(num_nodos+1)=hijo(j);
            
            %Llamada a la funcin para insertar en lista el nuevo nodo por orden de coste                                                
            [nodo,inicio,num_nodos]=insertar_en_lista(nuevo_coste,nodo,inicio,num_nodos);
            
            %Se marca con un '0' en el mapa el hijo para no volver a generarlo
            mapa(hijo(j).fila,hijo(j).columna)=0;
        end
        
        %Variacin del indice y del puntero al inicio para la siguiente iteracion
        indice=nodo(inicio).siguiente;
        inicio=indice;        
    end
end

%Una vez acabado el bucle, si se ha alcanzado la posicin objetivo
if(exito==1)    
    
    %Se inicializa el puntero i para almacenar el vector ruta
    i=nodo(indice).profundidad+1;
    tamano_ruta=i;
    
    %Bucle para almacenar la ruta
    while(i>0)
        
        %Se almacena ruta optima
        ruta(i).fila=nodo(indice).fila;
        ruta(i).columna=nodo(indice).columna;
        
        %Variacion del indice para la siguiente iteracion
        indice=nodo(indice).padre;
        
        %Actualizacin del puntero
        i=i-1;
    end 
    
    %Llamada a la funcin que almacena la ruta calculada en un fichero
    almacena_ruta_archivo(nfichero,ruta,tamano_ruta);
else
    
    %Si no se alcanza el objetivo, se advierte al usuario por pantalla 
    disp(sprintf('Exito no alcanzado :('));
end